from __future__ import print_function
__author__ = 'marco'

import numpy as np
from scipy.spatial import Delaunay
from scipy.spatial.distance import euclidean
from sklearn.neighbors import NearestNeighbors


def get_delaunay_edge_length_noise_threshold(ps, percentile, multiplier, verbose=False):
    tri = Delaunay(ps)
    edge_lengths = []
    for simplex in tri.simplices:
        # simplex like [p1, p2, p3], with p1, p2, p3 being indices in the original point set
        for edge in zip(simplex, simplex[1:]):
            edge_lengths.append(euclidean(ps[edge[0]], ps[edge[1]]))

    threshold = np.percentile(edge_lengths, percentile) * multiplier
    if verbose:
        print('Computing delauney edge length threshold:')
        print(str(percentile) + ' percentile: ' + str(np.percentile(edge_lengths, percentile)))
        print(str(percentile) + ' * ' + str(multiplier) + ' noise threshold: ' + str(threshold))
        print()
    return threshold


def get_nn_edge_length_noise_threshold(ps, percentile, multiplier, verbose=False):
    neighbors = NearestNeighbors(n_neighbors=2, algorithm='kd_tree').fit(ps)
    distances, _ = neighbors.kneighbors(ps)
    distances = distances[:, 1]  # first column of distances contains 0, as nn of each point is the point itself.

    threshold = np.percentile(distances, percentile) * multiplier
    if verbose:
        print('Computing nearest neighbor dist threshold:')
        print(str(percentile) + ' percentile: ' + str(np.percentile(distances, percentile)))
        print(str(percentile) + ' * ' + str(multiplier) + ' noise threshold: ' + str(threshold))
        print()
    return threshold
