__author__ = 'Alex'

from hierarchicalDBSCAN.ClusteringAlgorithms.minPtsHierarchicalDBScan import *
from hierarchicalDBSCAN.Datasets.DatasetLoader import *
from sklearn.metrics.cluster.supervised import homogeneity_completeness_v_measure
import numpy as np
import prettyplotlib as ppl
import hierarchicalDBSCAN.Clash.skeleton as sk

class DifDenseClashDescriptorDBSCAN:
    """ HDBSCAN based on Clustertree implementing edge quantile cut
    """
    def __init__(self, minPts):
        self.minPts = minPts
        self.db = minPtsHierarchicalDBScan(self.minPts)
        self.dataset = None


    def fit(self, dataset):
        """ run minPtsHierarchicalDBSCAN on the provided dataset

        :param dataset: dataset to be clustered
        """
        self.dataset = dataset
        self.db.fit(dataset)
        self.db.simplify_clustergraph()


    def predict(self, dataset, nr_clusters):
        """ tries different alpha cuts and reports the cut with the best density-based silhouette score

        :param dataset: dataset to be clustered in case fit was not used before
        :return: labeling of the best alpha cut
        """
        if self.dataset is None:
            self.fit(dataset)
        if nr_clusters is None:
            nr_clusters = len(dataset)*0.02

        densitychanges = list()

        for (s, t) in self.db.clustergraph.edges():
            pts_parent = dataset[list(self.db.clustergraph.node[s]['nodes']), :]
            _, _, polygons = sk.skeletonize(pts_parent, percentile=0.95)
            areaparent = polygons[0].area
            #densityparent = len(pts_parent) / areaparent

            pts_child = dataset[list(self.db.clustergraph.node[t]['nodes']), :]
            _, _, polygons = sk.skeletonize(pts_child)
            areachild = polygons[0].area
            #densitychild =  len(pts_child) / areachild

            #print("parent:", densityparent, "; areachild", densitychild)
            densitychanges.append((areaparent- areachild, self.db.clustergraph.node[t]["nodes"], self.db.clustergraph.node[t]["height"]))

        a = sorted(densitychanges, key=itemgetter(0), reverse=True)
        b = a[0:nr_clusters]
        c = sorted(b, key=itemgetter(2))

        # print("heights", [y for _,y in hierarchy_nodes])
        default = 1
        labels = [default] * len(dataset)
        for c_idx in range(len(c)):
            for p_idx in c[c_idx][1]:
                if labels[p_idx] == default:
                    labels[p_idx] = c_idx + 2
        return labels




if __name__ == "__main__":
    nr_clusters = 10
    dataset, labels_true = compound()

    difdense = DifDenseClashDescriptorDBSCAN(5)
    difdense.fit(dataset)

    labels_pred = difdense.predict(dataset, nr_clusters)

    print("q = ", homogeneity_completeness_v_measure(labels_true, labels_pred))
