__author__ = 'Alex'

import numpy as np
import skfuzzy as fuzz
from hierarchicalDBSCAN.Datasets.DatasetLoader import standard
import matplotlib.pyplot as plt


class FuzzyCMeans:
    """ Wrapper Class for Fuzzy-c-means for compatibility with sklearn
    """
    def __init__(self, nrClusters):
        self.k = nrClusters
        self.u = None

    def fit(self, dataset):
        """ fit fuzzy c-means with c = self.k to the dataset

        :param dataset: dataset to be clustered
        :return:
        """
        alldata = np.vstack((dataset[:, i] for i in range(dataset.shape[1])))
        cntr, self.u, u0, d, jm, p, fpc = fuzz.cluster.cmeans(alldata, self.k, 2, error=0.005, maxiter=100, init=None)

    def predict(self, dataset):
        """ returns the labeling of the dataset

        :param dataset: dataset to be clustered if fit() was not used before
        :return: cluster of maximal membership degree per point
        """
        if self.u is None:
            self.fit(dataset)
        return np.argmax(self.u, axis=0)


if __name__ == "__main__":
    dataset, labels_true = standard()
    fuzzy = FuzzyCMeans(nrClusters=3)
    fuzzy.fit(dataset)
    plt.scatter(dataset[:, 0], dataset[:, 1], c=fuzzy.predict())
    plt.show()
