__author__ = 'Alex'


import time

import numpy as np
import matplotlib.pyplot as plt
import prettyplotlib as ppl

import sklearn.cluster as cl
from sklearn.neighbors import kneighbors_graph
from sklearn.preprocessing import StandardScaler
from sklearn.metrics import homogeneity_completeness_v_measure
from hierarchicalDBSCAN.Robustness_tests import generate_dataset
from hierarchicalDBSCAN.Datasets.DatasetLoader import *
from hierarchicalDBSCAN.ClusterValidation.ExternalValidation import *

#algorithms
from hierarchicalDBSCAN.ClusteringAlgorithms.SingleLinkage import SingleLinkage
from hierarchicalDBSCAN.ClusteringAlgorithms.FuzzyCMeans import FuzzyCMeans
from hierarchicalDBSCAN.ClusteringAlgorithms.Optics import Optics
from hierarchicalDBSCAN.ClusteringAlgorithms.DifDenseHDBSCAN import DifDenseDBSCAN
from hierarchicalDBSCAN.ClusteringAlgorithms.Clique import TwoDimClique
from hierarchicalDBSCAN.ClusteringAlgorithms.alternatingOptimization import aoDBSCAN
from hierarchicalDBSCAN.Evaluation.Results.csv_to_latex import convert_csv_to_tex

np.random.seed(0)

blobs4d = generate_dataset("blobs", {"n_samples":100, "n_features":5, "random_state":None})
blobs1000d = generate_dataset("blobs", {"n_samples":100, "n_features":1000, "random_state":None})
moons = generate_dataset("moons", {"n_samples":300, "noise": 0.075, "random_state":None})
spiral = spiral()
compound = compound()
aggregation = aggregation()
t48k = t48k(size=2000)
flame = flame()
standard = standard()
r15 = r15()
jain = jain()

## example datasets
#dataset, labels = trex()
#dataset, labels_true = aggregation()
#dataset, labels_true
#dataset, labels_true = d31(size=1000)
#dataset, labels_true = jain()
#dataset, labels_true = pathbased()
#dataset, labels_true = spiral()

colors = np.array([x for x in 'bgrcmykbgrcmykbgrcmykbgrcmyk'])
colors = np.hstack([colors] * 20)


clustering_names = ['Mini-Batch-K-Means', 'Fuzzy-C-Means',
                    'Single Linkage', 'Complete Linkage', 'Wards Minimum Variance',
                    'OPTICS', 'CLIQUE', 'HDBSCAN(edge quantile)',
                    'aoDBSCAN(modsil)', 'aoDBSCAN(sil)', 'aoDBSCAN(edgecor)']

base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]*100

#plt.figure(figsize=(len(clustering_names) * 2 + 3, 9.5))
#plt.subplots_adjust(left=.02, right=.98, bottom=.001, top=.96, wspace=.05,
#                    hspace=.01)

plot_num = 1

nr_clusters = [15,3,2,3,6,3,7,2,2]
datasets = [r15, blobs1000d, moons, spiral, compound, standard, aggregation, flame, jain]
data_names = ["R15", "Blobs-1000D", "Moons", "Spiral", "Compound", "Standard", "Aggregation", "Flame", "Moons-DifDense"]
nr_clusters = [3,2,3,6,3,7,2,2]
datasets = [blobs1000d, moons, spiral, compound, standard, aggregation, flame, jain]
data_names = ["Blobs-1000D", "Moons", "Spiral", "Compound", "Standard", "Aggregation", "Flame", "Moons-DifDense"]

def write_csv(file_path, data, header=None):
    with open(file_path,"w") as f:
        if header is not None:
            f.write(",".join(header) + "\n")
        for line in data: f.write(",".join(line) + "\n")

def draw_labeling(data, labeling, true_labeling=False):
    labeling = np.asarray(labeling)
    if len(set(labeling)) > 1:
        colors = {label: [t/255 for t in base_colors[(i*3)]] for i, label in enumerate(set(labeling))}
        for label in set(labeling):
            filtered_dataset = data[np.where(labeling==label)[0].tolist(), ]
            if label == -1 and not true_labeling:
                ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                            color = [0.9, 0.9, 0.9], linewidth=1, marker='x', edgecolor='black')
            else:
                #filtered_dataset = dataset[np.where(labeling == label)[0].tolist(),:]
                ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                            color=colors[label])
    else:
        ppl.scatter(plt.gca(), data[:, 0], data[:, 1], label=str(1),
                    linewidth=1, marker='x', edgecolor='black')


for i_dataset, dataset in enumerate(datasets):
    print("dataset", data_names[i_dataset])

    X, y = dataset

    ### preprocessing
    # normalize dataset for easier parameter selection
    #X = StandardScaler().fit_transform(X)


    ### paramater estimation
    # estimate bandwidth for mean shift
    #bandwidth = cluster.estimate_bandwidth(X, quantile=0.3)
    # connectivity matrix for structured Ward
    connectivity = kneighbors_graph(X, n_neighbors=10, include_self=False)
    # make connectivity symmetric
    connectivity = 0.5 * (connectivity + connectivity.T)

    # create clustering estimators
    k_means = cl.MiniBatchKMeans(n_clusters=nr_clusters[i_dataset])
    fuzzy_c_means = FuzzyCMeans(nrClusters=nr_clusters[i_dataset])

    single = SingleLinkage(n_clusters=nr_clusters[i_dataset])
    complete = cl.AgglomerativeClustering(n_clusters=nr_clusters[i_dataset], linkage='complete')
    wards = cl.AgglomerativeClustering(n_clusters=nr_clusters[i_dataset], linkage='ward',
                                           connectivity=connectivity)
    #spectral = cluster.SpectralClustering(n_clusters=nr_clusters[i_dataset],
    #                                      eigen_solver='arpack')

    #birch = cluster.Birch(n_clusters=nr_clusters[i_dataset])
    optics = Optics(minPts=4)
    clique = TwoDimClique(threshold=4)
    hdbscan_edge_cut = DifDenseDBSCAN(minPts=4)


    ao_modsil = aoDBSCAN(opt="modsil")
    ao_sil = aoDBSCAN(opt="sil")
    ao_edge = aoDBSCAN(opt="edge")

    clustering_algorithms = [k_means, fuzzy_c_means,
                             single, complete, wards,
                             optics, clique, hdbscan_edge_cut,
                             ao_modsil, ao_sil, ao_edge]

    fig = plt.figure(figsize=(12, 16))
    plt.subplot(4, 3, 1)
    draw_labeling(X, y, True)
    plt.title("True Clustering", size=12)

    algo = 0
    results = []
    clustering = []
    for name, algorithm, subplot in zip(clustering_names, clustering_algorithms, range(2,len(clustering_names)+2)):
        print("algorithm", name)
        # predict cluster memberships
        t0 = time.time()
        algorithm.fit(X)
        t1 = time.time()
        if hasattr(algorithm, 'labels_'):
            y_pred = list(algorithm.labels_.astype(np.int))
        else:
            y_pred = algorithm.predict(X)

        ent = entropy(y, y_pred)
        pur = purity(y, y_pred)
        fmeas = fmeasure(y, y_pred)
        vmeas = homogeneity_completeness_v_measure(y, y_pred)[2]
        eval = homogeneity_completeness_v_measure(y, y_pred)
        results.extend((name,ent,pur,fmeas,vmeas))

        if subplot > 8:
            clustering.append((name, y_pred))

        # plot
        plt.subplot(4, 3, subplot)
        draw_labeling(X, y_pred)
        plt.title(name, size=12)

    fig.tight_layout()
    fig.subplots_adjust(hspace=0.3)
    fig.savefig('../../latex script/images/'+ data_names[i_dataset] +'.png')
    plt.close(fig)

    fig2 = plt.figure(figsize=(8, 8))
    for i, c in enumerate(clustering):
        plt.subplot(2, 2, i+1)
        draw_labeling(X, c[1])
        plt.title(c[0], size=12)
    fig2.tight_layout()
    fig2.subplots_adjust(hspace=0.3)
    fig2.savefig('../../latex script/images/'+ data_names[i_dataset] +'_detailed.png')
    plt.close(fig2)


    #test = np.rec.fromarrays((np.array(data_names), evaluation), names=("algorithm", "data"))
    dt=np.dtype('|S10,f4,f4,f4')
    evaluation = np.array(results)
    evaluation = evaluation.reshape(len(clustering_algorithms),5)
    print(evaluation)
    write_csv("Results/"+data_names[i_dataset]+"_eval.csv", evaluation, header=["algorithm","entropy", "purity", "fmeasure", "vmeasure"])
    convert_csv_to_tex(data_names[i_dataset])
