__author__ = 'Alex'
import prettyplotlib as ppl
import matplotlib.pyplot as plt
from hierarchicalDBSCAN.Datasets.DatasetLoader import *
from sklearn.metrics import pairwise_distances
from sklearn import datasets
import scipy.cluster.hierarchy as sch
from sklearn import cluster
import scipy.stats as stats
import math

iris = datasets.load_iris()
dataset = iris.data[:, 2:4]
labels_true = iris.target
labelmatrix_true = np.array(np.matrix([[labels_true[x] == labels_true[y] for y in range(len(labels_true))] for x in range(len(labels_true))]))


base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]*10


d = sch.distance.pdist(dataset)

distance_matrix = pairwise_distances(dataset)
distance_matrix = distance_matrix/distance_matrix.max()
similarity_matrix = -distance_matrix
similarity_matrix = similarity_matrix - similarity_matrix.min()

print("sim", similarity_matrix.shape)
print("comp", labelmatrix_true.shape)

fig = plt.figure(figsize=(13, 8))
xticks = np.arange(0, 151, 50)
xticks_dif = np.arange(0.5, len(labels_true) + 0.5)

#original dataset
plt.subplot(2, 3, 1)
ppl.scatter(plt.gca(), dataset[:, 0], dataset[:, 1], color='k', alpha=0.9)
plt.title("Iris data set", fontsize=12)


#true partition
plt.subplot(2, 3, 2)
k_means = cluster.MiniBatchKMeans(n_clusters=3)
k_means.fit(dataset)
labels_3 = k_means.predict(dataset)
labelmatrix_kmeans_3 = np.array(np.matrix([[labels_3[x] == labels_3[y] for y in range(len(labels_3))] for x in range(len(labels_3))]))

colors = {label: [t/255 for t in base_colors[(i*3)]] for i, label in enumerate(set(labels_3))}
for label in set(labels_3):
    #print("label", label)
    filtered_dataset = dataset[np.where(labels_3 == label), ][0]
    ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                color=colors[label], linewidth=1, edgecolor='black', alpha=0.9)
plt.xlabel('Petal length')
plt.ylabel('Petal width')
plt.title(r"k-Means Clustering (k=3), $\mathcal{C}_1$", fontsize=12)



#bad partition
plt.subplot(2, 3, 3)
k_means = cluster.MiniBatchKMeans(n_clusters=6)
k_means.fit(dataset)
labels_6 = k_means.predict(dataset)
labelmatrix_kmeans_6 = np.array(np.matrix([[labels_6[x] == labels_6[y] for y in range(len(labels_6))] for x in range(len(labels_6))]))

colors = {label: [t/255 for t in base_colors[(i*3)]] for i, label in enumerate(set(labels_6))}
for label in set(labels_6):
    #print("label", label)
    filtered_dataset = dataset[np.where(labels_6 == label), ][0]
    ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                color=colors[label], linewidth=1, edgecolor='black', alpha=0.9)
plt.xlabel('Petal length')
plt.ylabel('Petal width')
plt.title(r"k-Means Clustering (k=6), $\mathcal{C}_2$", fontsize=12)


#similarity matrix
plt.subplot(2, 3, 4)
ppl.pcolormesh(fig, plt.gca(), similarity_matrix)
plt.title(r"Similarity Matrix $sim$", fontsize=12)
plt.gca().set_xlim(0, len(labels_true))
plt.gca().set_xticks(xticks)
plt.gca().set_xticklabels(xticks)
plt.gca().set_yticks(xticks)
plt.gca().set_yticklabels(xticks)

#label matrix good
plt.subplot(2, 3, 5)
ppl.pcolormesh(fig, plt.gca(), labelmatrix_kmeans_3)

labelmatrix = pairwise_distances(np.transpose(np.matrix(labels_3)), metric="hamming")
score, _ = np.round(np.abs(stats.pearsonr(similarity_matrix.flatten(), labelmatrix.flatten())),2)

plt.title(r"Cluster Matrix $L_1$, $\rho(L_1,sim) = $"+str(score), fontsize=12)
plt.gca().set_xlim(0, len(labels_3))
plt.gca().set_xticks(xticks)
plt.gca().set_xticklabels(xticks)
plt.gca().set_yticks(xticks)
plt.gca().set_yticklabels(xticks)

#label matrix bad
plt.subplot(2, 3, 6)
ppl.pcolormesh(fig, plt.gca(), labelmatrix_kmeans_6)

labelmatrix = pairwise_distances(np.transpose(np.matrix(labels_6)), metric="hamming")
score, _ = np.round(np.abs(stats.pearsonr(similarity_matrix.flatten(), labelmatrix.flatten())),2)

plt.title(r"Cluster Matrix $L_2$, $\rho(L_2,sim) = $"+str(score), fontsize=12)
plt.gca().set_xlim(0, len(labels_6))
plt.gca().set_xticks(xticks)
plt.gca().set_xticklabels(xticks)
plt.gca().set_yticks(xticks)
plt.gca().set_yticklabels(xticks)

fig.tight_layout()

fig.savefig('../../latex script/images/similarity-matrices.png')
plt.show()
