__author__ = 'Alex'
from hierarchicalDBSCAN.ClusteringAlgorithms.SingleLinkage import *
import numpy as np
import matplotlib.pyplot as plt
import prettyplotlib as ppl
from sklearn import datasets
from sklearn import cluster


np.random.seed(2)

# import some data to play with
iris = datasets.load_iris()
dataset = iris.data[:, 2:4]  # we only take the first two features.
Y = iris.target

fig = plt.figure(figsize=(12, 3))
plt.clf()

# Plot the training points

base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]


plt.subplot(1, 3, 1)

ppl.scatter(dataset[:, 0], dataset[:, 1], color='black', alpha=1)
plt.xlabel('Petal length')
plt.ylabel('Petal width')

plt.subplot(1, 3, 3)
k_means = cluster.MiniBatchKMeans(n_clusters=3)
k_means.fit(dataset)

labels = k_means.predict(dataset)
colors = {label: [t/255 for t in base_colors[i*3]] for i, label in enumerate(set(labels))}
for label in set(labels):
    #print("label", label)
    filtered_dataset = dataset[np.where(labels == label), ][0]
    ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                color=colors[label], linewidth=1, edgecolor='black', alpha=0.5)
plt.xlabel('Petal length')
plt.ylabel('Petal width')

plt.subplot(1, 3, 2)
colors = {label: [t/255 for t in base_colors[i*3]] for i, label in enumerate(set(Y))}
for label in set(Y):
    filtered_dataset = dataset[np.where(Y == label), ][0]
    ppl.scatter(plt.gca(), filtered_dataset[:, 0], filtered_dataset[:, 1], label=str(label),
                color=colors[label], linewidth=1, edgecolor='black', alpha=0.5)
plt.xlabel('Petal length')
plt.ylabel('Petal width')

fig.tight_layout()
fig.savefig('../../latex script/images/iris-example-k-means.png')
plt.show()