__author__ = 'Alex'
from hierarchicalDBSCAN.ClusteringAlgorithms.Optics import *
from hierarchicalDBSCAN.Datasets.DatasetLoader import *
from hierarchicalDBSCAN.Robustness_tests import generate_dataset
import matplotlib.pyplot as plt
import prettyplotlib as ppl

if __name__ == "__main__":
    np.random.seed(17041991)

    dataset = generate_dataset("blobs", {"n_samples":30, "random_state":None})
    reachability = np.array([])

    labels = [0]*10 + [1]*10 + [2]*10
    base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]
    colors = {label: [t/255 for t in base_colors[i*3]] for i, label in enumerate(set(labels))}

    fig = plt.figure(figsize=(12, 4))
    plt.subplot(1,2,1)

    x = np.array([[0.4,0.8],
                  [0.5,1.2],
                  [0.6,0.5],
                  [0.6,0.5],
                  [0.7,1.15],
                  [0.8,0.65],
                  [1.0,0.5],
                  [1.1,0.85],
                  [1.2,0.11],
                  [0.75, 0.75]])
    ppl.scatter(x[:,0], x[:,1], color=colors[0], alpha=0.8, s=50)

    x = np.array([[0.7,2.0],
                  [0.7,2.3],
                  [1.1,1.75],
                  [1.1,2.2],
                  [1.3,2.3],
                  [1.40,2.05],
                  [1.43,2.3],
                  [1.4,2.6],
                  [1.5,1.6],
                  [1.6,1.8]])
    ppl.scatter(x[:,0], x[:,1], color=colors[1], alpha=0.8, s=50)

    x = np.array([[1.7,0.6],
                  [1.75,1.3],
                  [2.0,0.9],
                  [2.1,1.1],
                  [2.15,1.5],
                  [2.25,1.35],
                  [2.4,1.05],
                  [2.45,0.65],
                  [2.6,1.4],
                  [2.8,1.0]])
    ppl.scatter(x[:,0], x[:,1], color=colors[2], alpha=0.8, s=50)


    plt.subplot(1,2,2)
    n = [0.3, 0.325, 0.35, 0.4, 0.475, 0.56, 0.68, 0.73, 0.82, 0.98]*3
    n += (np.random.rand(len(n))/7)
    n = sorted(n[0:10]) + sorted(n[10:20]) + sorted(n[20:30])

    ax = plt.gca()
    ppl.bar(ax, np.arange(10), n[0:10], color=colors[0])
    ppl.bar(ax, np.arange(10)+10, n[10:20], color=colors[1])
    ppl.bar(ax, np.arange(10)+20, n[20:30], color=colors[2])
    ax.set_xlim(0, 30)
    fig.savefig('../../latex script/images/optics-reachability-plot.png')

    plt.show()