__author__ = 'Alex'

import numpy as np
from sklearn.cluster import DBSCAN
import matplotlib.pyplot as plt
from sklearn.datasets import make_moons
from hierarchicalDBSCAN.Robustness_tests import generate_dataset
from hierarchicalDBSCAN.Datasets.DatasetLoader import *
import colorsys
import random

def dbscan(dataset, eps=0.3, min_samples=10):
    db = DBSCAN(eps=eps, min_samples=min_samples).fit(dataset)
    core_samples_mask = np.zeros_like(db.labels_, dtype=bool)
    core_samples_mask[db.core_sample_indices_] = True
    return db.labels_, core_samples_mask

def plot(dataset, labels, name, values):
    max_labels = len(set(labels))
    hsv_tuples = [(x * 1.0 / max_labels, 0.7, 0.9) for x in range(max_labels)]
    rgb_tuples = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    random.shuffle(rgb_tuples)
    #plt.gca().axis("equal")
    plt.title("{} (eps={}, minPts={})".format(name, values[0], values[1]))
    for label in set(labels):
        #print("label", label)
        filtered_dataset = dataset[np.where(labels == label), ][0]

        if label < 0:
            # plot noise
            plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c='black', s=50, marker='x')
            test =0
        else:
            # plot points in a cluster
            plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c=rgb_tuples[label], s=50)


def example_dbscan_parameters():
    fig = plt.figure(figsize=(11, 15))

    plt.subplot(4, 2, 1)
    np.random.seed(17041991)
    dataset, labels_true = aggregation()
    values = [1.451, 1]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Aggregation", values)

    plt.subplot(4, 2, 2)
    np.random.seed(17041991)
    dataset, labels_true = generate_dataset("moons", {"n_samples": 100, "noise": 0.075, "random_state":None})
    values = [0.324, 3]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Moons", values)

    plt.subplot(4, 2, 3)
    np.random.seed(17041991)
    dataset, labels_true = generate_dataset("blobs", {"n_samples": 100, "n_features": 1000, "random_state":None})
    values = [141.315, 5]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Blobs-1000D", values)

    plt.subplot(4, 2, 4)
    np.random.seed(17041991)
    dataset, labels_true = spiral()
    values = [1.107, 1]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Spirals", values)

    plt.subplot(4, 2, 5)
    np.random.seed(17041991)
    dataset, labels_true = r15()
    values = [0.625, 1]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "R15", values)

    plt.subplot(4, 2, 5)
    np.random.seed(17041991)
    dataset, labels_true = r15()
    values = [0.422, 5]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "R15", values)

    plt.subplot(4, 2, 6)
    np.random.seed(17041991)
    dataset, labels_true = d31(size=1000)
    values = [1.025, 10]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "D31", values)

    plt.subplot(4, 2, 7)
    np.random.seed(17041991)
    dataset, labels_true = flame()
    values = [0.992, 6]
    labels, _ = dbscan(dataset=dataset, eps=values[0], min_samples=values[1])
    plot(dataset, labels, "Flame", values)

    plt.show()


if __name__ == "__main__":
    ## example datasets
    #dataset, labels = trex()
    #dataset, labels_true = aggregation()
    #dataset, labels_true = compound()
    #dataset, labels_true = r15()
    #dataset, labels_true = d31()
    #dataset, labels_true = jain()
    #dataset, labels_true = pathbased()
    #dataset = t48k(size=4000)
    #dataset, labels_true = spiral()
    #dataset, labels_true = flame()


    example_dbscan_parameters()
