__author__ = 'Alex'

import numpy as np
from sklearn.cluster import DBSCAN
import matplotlib.pyplot as plt
from sklearn.datasets import make_moons
import colorsys
from numpy import genfromtxt
from sklearn.metrics import pairwise_distances

def dbscan(dataset, eps=0.3, min_samples=10):
    db = DBSCAN(eps=eps, min_samples=min_samples).fit(dataset)
    core_samples_mask = np.zeros_like(db.labels_, dtype=bool)
    core_samples_mask[db.core_sample_indices_] = True
    return db.labels_, core_samples_mask

def example_dbscan_parameters():
    dataset = genfromtxt('fig-dbscan-basics.txt', delimiter=' ')
    distance_matrix = pairwise_distances(dataset)

    #dbscan(dataset=)
    params = (0.5, 5)

    labels, _ = dbscan(dataset=dataset, eps=params[0], min_samples=params[1])
    labels = np.array([-1 if labels[x] == -1 else 0 if is_border(distance_matrix[x,:], params[0], params[1]) else 1 for x in range(len(dataset))])
    print(labels)

    fig = plt.figure(figsize=(15, 7))

    hsv_tuples = [(x * 1.0 / 2, 0.7, 0.9) for x in range(2)]
    rgb_tuples = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))

    plt.title("DBSCAN (eps={}, minPts={})".format(params[0], params[1]))
    for label in set(labels):
        print("label", label)
        filtered_dataset = dataset[np.where(labels == label), ][0]
        print(filtered_dataset)
        if label == -1:
            # plot noise
            plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c='black', s=50, marker='x')
        else:
            # plot points in a cluster
            plt.scatter(filtered_dataset[:, 0], filtered_dataset[:, 1], c=rgb_tuples[label], s=50)
    plt.gca().axis("equal")

        # plot points outside a cluster
    plt.show()

def is_border(A,eps, minPts):
    return len([n for n in range(len(A)) if A[n] <= eps]) >= minPts


if __name__ == "__main__":
    example_dbscan_parameters()