__author__ = 'Alex'

import numpy as np
from sklearn.cluster import DBSCAN
import matplotlib.pyplot as plt
from hierarchicalDBSCAN.Robustness_tests import generate_dataset
import logging
import networkx as nx
import colorsys
import math
import matplotlib.colors as cls
from hierarchicalDBSCAN.ClusteringAlgorithms.DendrogramTools import labels_for_cut_height
from scipy.stats import rankdata
from networkx.drawing.nx_agraph import graphviz_layout

def dbscan(dataset, eps=0.3, min_samples=10):
    db = DBSCAN(eps=eps, min_samples=min_samples).fit(dataset)
    core_samples_mask = np.zeros_like(db.labels_, dtype=bool)
    core_samples_mask[db.core_sample_indices_] = True
    return db.labels_, core_samples_mask


def draw_pie(ax, ratios=[0.4, 0.3, 0.3], X=0, Y=0, size=1000):
    N = len(ratios)

    xy = []

    start = 0.25
    for ratio in ratios:
        x = [0] + np.cos(np.linspace(2*math.pi*start,2*math.pi*(start+ratio), 30)).tolist()
        y = [0] + np.sin(np.linspace(2*math.pi*start,2*math.pi*(start+ratio), 30)).tolist()
        xy1 = list(zip(x,y))
        xy.append(xy1)
        start += ratio

    pos = 0

    colors = [cls.ColorConverter().to_rgb("#B2FF66"), cls.ColorConverter().to_rgb("#6666FF")]
    #hex code green : B2FF66
    #hex code blue  : 6666FF
    for i, xyi in enumerate(xy):
        ax.scatter([X],[Y] , marker=(xyi,0), s=size, vmin=0, vmax=len(ratios)-1, c=colors[i])
        #pos +=1


def set_node_size(currentgraph, n, N, max_size=500):
    currentgraph.node[n]["size"] = len(currentgraph.node[n]["nodes"])/N*max_size
    currentgraph.node[n]["ratio"] = len(currentgraph.node[n]["cores"])/len(currentgraph.node[n]["nodes"])


def draw_advanced_dbscan(dataset, clustergraph, use_height=True, color_edges = False):
    pos = graphviz_layout(clustergraph ,prog='dot')
    N = len(dataset)

    if use_height:
        pos = {x:(y[0], clustergraph.node[x]["height"]) for (x, y) in pos.items()}
    [set_node_size(clustergraph, n,N) for n in clustergraph.node]

    if color_edges:
        #colors = rankdata([clustergraph.node[u]["height"] - clustergraph.node[v]["height"]
         #                  for (u, v) in clustergraph.edges()], method="max")
        colors = [clustergraph.node[u]["height"] - clustergraph.node[v]["height"]
                  for (u, v) in clustergraph.edges()]
        nx.draw_networkx_edges(clustergraph, pos, arrows=False, edge_color=colors, edge_cmap=plt.cm.cool, width=3)
    else:
        nx.draw_networkx_edges(clustergraph, pos, arrows=False)

    ax = plt.gca()
    [draw_pie(ax, [1-clustergraph.node[n]["ratio"], clustergraph.node[n]["ratio"]], pos[n][0], pos[n][1], clustergraph.node[n]["size"]) for n in clustergraph.node]

    if color_edges:
        return [min(colors), max(colors)]
    #nx.draw_networkx_labels(clustergraph,pos, font_size=16)


def draw_clustered_scatter(dataset, clustergraph, cut_level=None):
    if cut_level is  None:
        clusters=[-1]*len(dataset)
    else:
        clusters = labels_for_cut_height(len(dataset), clustergraph, cut_level)


    n = len(set([c for c in clusters if c >= 0]))

    hsv_tuples = [(x * 1.0 / n, 0.9, 0.5) for x in range(n)]
    rgb_tuples = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))

    logging.debug("{0}".format(clusters))

    color_list = [rgb_tuples[c] if c >-1 else (0,0,0) if c == -1 else (0.5,0.5,0.5)  for c in clusters]
    p = plt.scatter(
        dataset[:, 0], dataset[:, 1], marker="o", color=color_list, s=50)
    plt.gca().axis("equal")
    return p


if __name__ == "__main__":

    np.random.seed(17041991)
    dataset = generate_dataset("blobs", {"n_samples":1000,  "random_state":None})
