__author__ = 'Alex'

from hierarchicalDBSCAN.ClusteringAlgorithms.minPtsHierarchicalDBScan import *
from hierarchicalDBSCAN.ClusteringAlgorithms.epsDBScan import *
from hierarchicalDBSCAN.Datasets.DatasetLoader import *
import prettyplotlib as ppl

class interactiveAODBSCAN:
    def __init__(self, dataset):
        self.dataset = dataset
        self.n = len(dataset)
        self.dist_mat = pairwise_distances(dataset)

        self.minPtsDBS = minPtsHierarchicalDBScan(2)
        self.epsDBS = epsHierarchicalDBScan(1)
        self.minPtsSlider = None
        self.epsSlider = None
        self.updating = False
        self.lineeps = None
        self.lineminPts = None

    def run_eps(self, eps):
        self.eps = eps
        self.epsDBS.configure(self.eps)
        self.epsDBS.fit(self.dataset, dist_mat=self.dist_mat)
        self.epsDBS.simplify_clustergraph()


    def run_minPts(self, minPts):
        self.minPts = minPts
        self.minPtsDBS.configure(self.minPts)
        self.minPtsDBS.fit(self.dataset, dist_mat=self.dist_mat)
        self.minPtsDBS.simplify_clustergraph()


    def plot_fixedEpsDendrogram(self, eps):
        plt.subplot(1, 3, 1)
        plt.cla()

        self.run_eps(eps)
        draw_advanced_dbscan(self.dataset, self.epsDBS.simplified_clustergraph)
        plt.tick_params(
            axis='x',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            bottom='off',      # ticks along the bottom edge are off
            top='off',         # ticks along the top edge are off
            labelbottom='off') # labels along the bottom edge are off
        plt.tick_params(
            axis='y',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            left='on',      # ticks along the bottom edge are off
            right='off')         # ticks along the top edge are off
        ticks, labels = plt.yticks()
        labels = [self.n-tick for tick in ticks]
        plt.yticks(ticks, labels)
        plt.title("HDBSCAN dendrogram for eps={0:.2f}".format(eps))

        if self.minPtsSlider is None:

            #draw slider
            ylim = [min(labels), max(labels)]
            axcolor = 'lightgoldenrodyellow'
            ax_minPtsslider = plt.axes([0.05, 0.1, 0.02, 0.8], axisbg=axcolor)
            init_cut_height = ylim[0] + ((ylim[1]-ylim[0])/2)
            self.minPtsSlider = VertSlider(ax_minPtsslider, '', ylim[0], ylim[1],
                                           valinit=init_cut_height, dragging=False, rounding=True, reverse=True)
            plt.subplot(1,3,1)
            self.lineminPts = plt.axhline(self.n - self.minPtsSlider.val)  # l = horizontal line which has to be updated
            print("init line", self.lineminPts.get_ydata())

            #define update of scatter plot
            def update(val):
                plt.figtext(x = 0.050, y = 0.048, s="minPts")
                # update horizontal line in tree plot
                plt.subplot(1,3,1)
                self.lineminPts.set_ydata([self.n - val, self.n - val])

                #calculate cluster colors
                labels = self.epsDBS.predict_cut_height(self.n - val)
                self.labels = labels
                logging.info("final clusters: {0}".format(labels))

                n = len(set(labels))
                if -1 in labels:
                    n -= 1

                hsv_tuples = [(x * 1.0 / n, 0.5, 0.5) for x in range(n)]
                rgb_tuples = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
                rgb_tuples.append((0.5,0.5,0.5))

                logging.debug("{0}".format(labels))
                #l != -1 => cluster, l = -2 => border of more than one cluster, l = -1 => not in any cluster
                color_list = [rgb_tuples[l] if l > -1 else (0.1,0.1,0.1) if l == -1 else (0.5,0.5,0.5) for l in labels]
                self.scatter.set_color(color_list)
                plt.subplot(1,3,2)
                plt.title("Clustering result eps = {0}, minPts = {1}".format(round(self.eps,2), round(val,0)))

                plt.gcf().canvas.draw()

            self.minPtsSlider.on_changed(update)
            update(self.minPtsSlider.val)
        else:
            #ylim = plt.gca().get_ylim()
            ylim = [min(labels), max(labels)]
            self.minPtsSlider.valmin = ylim[1]
            self.minPtsSlider.valmax = ylim[0]
            self.minPtsSlider.ax.set_ylim((ylim[1], ylim[0]))
            val = ylim[0] + ((ylim[1]-ylim[0])/2)
            self.minPtsSlider.valinit = val
            self.minPtsSlider.set_val(val)
            self.minPtsSlider.vline.set_ydata([self.n - val, self.n - val])
        self.updating = False


    def plot_fixedMinPtsDendrogram(self, minPts):
        plt.subplot(1, 3, 3)
        plt.cla()

        self.run_minPts(minPts)
        draw_advanced_dbscan(self.dataset, self.minPtsDBS.simplified_clustergraph)

        plt.tick_params(
            axis='x',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            bottom='off',      # ticks along the bottom edge are off
            top='off',         # ticks along the top edge are off
            labelbottom='off') # labels along the bottom edge are off
        plt.tick_params(
            axis='y',          # changes apply to the x-axis
            which='both',      # both major and minor ticks are affected
            left='off',      # ticks along the bottom edge are off
            right='on')         # ticks along the top edge are off
        plt.gca().yaxis.tick_right()
        plt.title("HDBSCAN dendrogram for minPts={0:.0f}".format(minPts))

        if self.epsSlider is None:
            plt.figtext(x = 0.928, y = 0.048, s="eps")
            #draw slider
            ylim = plt.gca().get_ylim()
            axcolor = 'lightgoldenrodyellow'
            ax_epsSlider = plt.axes([0.925, 0.1, 0.02, 0.8], axisbg=axcolor)
            init_cut_height = ylim[0] + ((ylim[1]-ylim[0])/2)
            self.epsSlider = VertSlider(ax_epsSlider, '', ylim[0], ylim[1], valinit=init_cut_height, dragging=False)
            plt.subplot(1,3,3)
            self.lineeps = plt.axhline(self.epsSlider.val)  # l = horizontal line which has to be updated

            #define update of scatter plot
            def update(val):
                # update horizontal line in tree plot
                self.lineeps.set_ydata([val, val])

                #calculate cluster colors
                labels = self.minPtsDBS.predict_cut_height(val)
                self.labels = labels
                logging.info("final clusters: {0}".format(labels))

                n = len(set(labels))
                if -1 in labels:
                    n -= 1

                hsv_tuples = [(x * 1.0 / n, 0.5, 0.5) for x in range(n)]
                rgb_tuples = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
                rgb_tuples.append((0.5,0.5,0.5))

                logging.debug("{0}".format(labels))
                #l != -1 => cluster, l = -2 => border of more than one cluster, l = -1 => not in any cluster
                color_list = [rgb_tuples[l] if l > -1 else (0.1,0.1,0.1) if l == -1 else (0.5,0.5,0.5) for l in labels]
                self.scatter.set_color(color_list)
                plt.subplot(1,3,2)
                plt.title("Clustering result eps = {0}, minPts = {1}".format(round(val,2), round(self.minPts,0)))

                plt.gcf().canvas.draw_idle()

            self.epsSlider.on_changed(update)
            update(self.epsSlider.val)
        else:
            ylim = plt.gca().get_ylim()
            self.epsSlider.valmin = ylim[0]
            self.epsSlider.valmax = ylim[1]
            self.epsSlider.ax.set_ylim((ylim[0], ylim[1]))
            val = ylim[0] + ((ylim[1]-ylim[0])/2)
            self.epsSlider.valinit = val
            self.epsSlider.set_val(val)
            self.epsSlider.vline.set_ydata([val, val])



        self.updating = False



    def plot_scatter(self, labels=None):
        plt.subplot(1, 3, 2)

        if labels is None:
            scatter = plt.scatter(
                self.dataset[:, 0], self.dataset[:, 1], marker="o", color="b", s=50,
                edgecolor=[0.7, 0.7, 0.7], alpha= 0.5)
        else:
            base_colors = [(255,187,120),(255,127,14),(174,199,232),(44,160,44),(31,119,180),(255,152,150),(214,39,40),
               (197,176,213),(152,223,138),(148,103,189),(247,182,210),(227,119,194),(196,156,148),(140,86,75),
               (127,127,127),(219,219,141),(199,199,199),(188,189,34),(158,218,229),(23,190,207)]*100

            rgb_tuples  = {label: [t/255 for t in base_colors[(i)]] for i, label in enumerate(set(labels))}
            color_list = [rgb_tuples[label] if l > -1 else (0.1,0.1,0.1) if label == -1 else (0.7,0.7,0.7) for label in labels]
            scatter.set_color(color_list)
        plt.gca().axis("equal")
        return scatter


    def plot(self, eps=None, minPts=None, always_update=False):

        #general figure
        fig = plt.figure(figsize=(21, 8))
        ax = plt.gca()
        plt.subplots_adjust(left=0.1, bottom=0.1)

        self.scatter = self.plot_scatter()

        #plot either minPtsDendrogram or epsDendrogram
        if eps is not None:
            self.plot_fixedEpsDendrogram(eps)
        else:
            self.plot_fixedMinPtsDendrogram(minPts)


        axcolor = 'lightgoldenrodyellow'
        #reset minPts
        ax_reseteps = plt.axes([0.10, 0.025, 0.1, 0.04])
        button_reset_minPts = Button(ax_reseteps, 'Reset minPts', color=axcolor, hovercolor='0.975')

        #reset eps
        ax_resetminPts = plt.axes([0.8, 0.025, 0.1, 0.04])
        button_reset_eps = Button(ax_resetminPts, 'Reset eps', color=axcolor, hovercolor='0.975')

        #fix minPts
        ax_transferminPts = plt.axes([0.382, 0.025, 0.1, 0.04])
        button_transferminPts = Button(ax_transferminPts, 'Transfer minPts -->', color=axcolor, hovercolor='0.975')

        #fix eps
        ax_transfereps = plt.axes([0.518, 0.025, 0.1, 0.04])
        button_transfereps = Button(ax_transfereps, '<-- Transfer eps', color=axcolor, hovercolor='0.975')

        def reset_eps(event):
            if self.epsSlider is not None:
                self.epsSlider.reset()
        button_reset_eps.on_clicked(reset_eps)

        def reset_minPts(event):
            if self.minPtsSlider is not None:
                self.minPtsSlider.reset()
        button_reset_minPts.on_clicked(reset_minPts)

        def fix_eps(event):
            if self.updating == False:
                self.updating = True
                self.plot_fixedEpsDendrogram(self.epsSlider.val)
            else:
                print("please wait for updated results")
        button_transfereps.on_clicked(fix_eps)

        def fix_minPts(event):
            if self.updating == False:
                self.updating = True
                self.plot_fixedMinPtsDendrogram(self.minPtsSlider.val)
            else:
                print("please wait for updated results")
        button_transferminPts.on_clicked(fix_minPts)
        plt.show()

if __name__ == "__main__":
    X, y = spiral()

    iao = interactiveAODBSCAN(X)
    iao.plot(minPts=2)
